// may want to consider using jquery-color:
// https://github.com/jquery/jquery-color/blob/master/jquery.color.js
function color(color_str)
{
    this._color_str = color_str.replace(/ /g,'');
}
color.prototype.is_transparent = function() {
    if (this._color_str == 'transparent') return true;
    if (this._color_str == 'rgba(0,0,0,0)') return true;

    // HACK: temporary until regex are setup
    if (this._color_str == 'rgb(255,255,0)') return false;

    console.error('unable to parse color: ' + this._color_str)

    return false;
}

/**
 * Color the rows, where each day in the week has the same color.
 *
 * @class_identifier:
 *     class of table to apply coloring (i.e., used by this function to identify the intended table)
 *
 * @nr_row_span:
 *     the number of sequential rows that should have the same color
 */
function table_coloring(class_identifier, nr_rows_in_group)
{
    // table row coloring
    var color_header = '#CCCC99';
    var color_group = [ '#E0E0E0', '#FFCC99' ];
    var nr_colors = color_group.length;
    //var nr_rows_in_group = 1;
    //var class_identifier = 'grades';

    truncate = function (number) {
        return Math[number < 0 ? 'ceil' : 'floor'](number);
    };

    // table headers
    $('table.' + class_identifier + ' > thead > tr').each(function(index) {
        $(this).css('background-color', color_header);
    });

    // all rows except the header
    $('table.' + class_identifier + ' > tbody > tr').each(function(index) {
        // do not override any existing color
        if (new color($(this).css('background-color')).is_transparent()) {
            $(this).css('background-color', color_group[truncate(index/nr_rows_in_group)%nr_colors]);
        }
    });
}

/**
 * Add an additional column displaying the week number.
 */
function table_group_column(class_identifier, nr_rows_in_group)
{
    truncate = function (number) {
        return Math[number < 0 ? 'ceil' : 'floor'](number);
    };

    // table headers
    $('table.' + class_identifier + ' tr:first').each(function(index) {
        $(this).prepend("<th>Week</th>");
        $(this).find('td:first').addClass('date');
        //$(this).find('td:first').css('text-align', 'center');
    });

    // all rows except the header
    rows = $('table.' + class_identifier + ' tr:gt(0)');
    total_groups = truncate($(rows).length/nr_rows_in_group) + 1;
    $(rows).each(function(index) {
        // only add td at the beginning of a group of rows
        if ((index % nr_rows_in_group) == 0) {
            var group_nr = truncate(index/nr_rows_in_group) + 1;
            var nr_row_span = nr_rows_in_group;

            if (group_nr == total_groups) {
                // last set of rows may not have nr_rows_in_group rows, so
                // change the rowspan appropriately
                nr_row_span = $(rows).length % nr_rows_in_group;
            }

            // add the new entry for the addition of the column
            $(this).prepend('<td style="text-align:center" rowspan=' + nr_row_span + '>'
                + group_nr + '</td>');
        }
    });
}

/**
 * date_col
 *  - 1-based index of col containing date string
 */
function table_week_coloring(tbl_class, date_col)
{
    // table row coloring
    var color_header = '#CCCC99';
    var color_group = [ '#E0E0E0', '#FFCC99' ];
    var nr_colors = color_group.length;

    // add week column to table header
    $('table.' + tbl_class + ' tr:first').each(function(index) {
        $(this).css('background-color', color_header);

        // make width small (and will autofit the width of 'Week'
        $(this).prepend('<th style="width: 1%;">Week</th>');
        $(this).find('td:first').addClass('date');
        //$(this).find('td:first').css('text-align', 'center');
    });

    Date.prototype.getWeek = function() {
        var jan_one = new Date(this.getFullYear(), 0, 1);
        return Math.ceil(((((this - jan_one))/86400000) + jan_one.getDay() + 1)/7);
    };

    Date.prototype.getAbbrevMonth = function() {
        var monthStrings = [
            "Jan", "Feb", "Mar", "Apr", "May", "June",
            "July", "Aug", "Sept", "Oct", "Nov", "Dec"];

        return monthStrings[this.getMonth()];
    };

    // all rows except the header
    rows = $('table.' + tbl_class + ' tr:gt(0)');
    curr_wk = -1;
    first_wk_nr = -1;
    weeks_cnt = 1;
    cnt_per_wk = 0;
    // only add td at the beginning of a group of rows (i.e., first row of the week)
    first_row_of_wk = $(rows).eq(0);
    $(rows).each(function(index) {
        date_cell = $(this).find('td').eq(date_col)
        txt_date = date_cell.text();

        // find only one of multiple dates (e.g., 8-10 September returns 10 September)
        dtMatch = txt_date.match(/^(.*?)(\d+\s*)([a-zA-Z]+)\s*$/);
        txt_date = dtMatch[2] + dtMatch[3];

        // get a date object representing the value in the date cell
        var today = new Date();
        var year = today.getFullYear();
        txt_split = txt_date.split(" ");
        parsed_date = new Date(txt_split[1] + " " + txt_split[0] + " " + year.toString());

        // use abbreviated month name
        date_cell.text(dtMatch[1] + dtMatch[2] + parsed_date.getAbbrevMonth());

        // first iteration (i.e., row)
        if (curr_wk == -1) {
            first_wk_nr = curr_wk = parsed_date.getWeek();
        }

        if (curr_wk != parsed_date.getWeek()) {
            // curr row is for a new week (i.e., transition to new week)
            // add the new entry (i.e., cell of new column)
            $(first_row_of_wk).prepend('<td style="text-align:center" rowspan=' + cnt_per_wk + '>'
                + (curr_wk - first_wk_nr + 1) + '</td>');

            // reset week information
            cnt_per_wk = 1;
            curr_wk = parsed_date.getWeek();
            weeks_cnt++;
            first_row_of_wk = $(this);

            // set background color for current row
            // do not override any existing colors (i.e., background color is not transparent)
            if (new color($(this).css('background-color')).is_transparent()) {
                $(this).css('background-color', color_group[weeks_cnt % nr_colors]);
            }
        } else {
            // have not encountered a transition from curr week to the next
            cnt_per_wk++;

            // set background color for current row
            // do not override any existing color
            if (new color($(this).css('background-color')).is_transparent()) {
                $(this).css('background-color', color_group[weeks_cnt % nr_colors]);
            }
        }
    });

    // Add cell for week column (i.e., finish processing rows for the last week)
    if (cnt_per_wk != 0) {
         // add the new entry for the addition of the column
         $(first_row_of_wk).prepend('<td style="text-align:center" rowspan=' + cnt_per_wk + '>'
            + (curr_wk - first_wk_nr + 1) + '</td>');
    }
}

/**
 * Center text of each cell in a table's column.
 *
 * column_nr:
 *      1-based column index
 */
function table_col_center(class_identifier, column_nr)
{
    // table headers
    $('table.' + class_identifier + ' tr:first').each(function(index) {
        //$(this).find('td:first').css('text-align', 'center');
    });

    // all rows except the header
    $('table.' + class_identifier + ' > tbody > tr > td:nth-child(' + column_nr + ')').each(function(index) {
        $(this).css('text-align', 'center');
        //$(this).children("td:nth-child(column_nr)").css('text-align', 'center');
    });
    // above can be done in one line
    //$('table.' + class_identifier + ' > tbody > tr > td:nth-child(2)').css('text-align', 'center');
}
