#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <elevator_calls.h>

#define MIN_TYPE 0
#define MAX_TYPE 3
#define MIN_FLOOR 1
#define MAX_FLOOR 10
#define PERCENT_TO_FLOOR_ONE 80

#define SEED 39
#define NUM_PEOPLE 10000
#define WAIT_TIME (5 * 60)

int time_diff(struct timeval *t1, struct timeval *t2, struct timeval *ret) {
	long long sec, usec;
	sec = t2->tv_sec - t1->tv_sec;
	usec = t2->tv_usec - t1->tv_usec;
	while (usec < 0) {
		sec -= 1;
		usec += 1000000;
	}

	ret->tv_sec = sec;
	ret->tv_usec = usec;
	return 0;
}

int my_rand(int min, int max) {
	return (rand() % (max-min)) + min;
}
int rand_type() {
	return my_rand(MIN_TYPE, MAX_TYPE);
}
int rand_start_floor() {
	return my_rand(MIN_FLOOR, MAX_FLOOR);
}
int rand_dest_floor(int start_floor) {
	int temp, ret;
	if (start_floor == MIN_FLOOR) {
		ret = MAX_FLOOR - MIN_FLOOR + 1;
	}
	else {
		temp = my_rand(0, 100);
		if (temp <= 80) {
			ret = MIN_FLOOR;
		}
		else {
			ret = my_rand(MIN_FLOOR+1, MAX_FLOOR-1);
			if (ret == start_floor) {
				ret = MAX_FLOOR;
			}
		}
	}
	return ret;
}

int main(int argc, char **argv) {
	struct timeval t1, t2, diff;
	int i, ret, type, start, dest;
	long long sleep_time;
	
	if (argc != 1) {
		fprintf(stderr, "Wrong number of arguments!\n");
		fprintf(stderr, "  call using %s\n", argv[0]);
		return -1;
	}
	srand((unsigned) SEED);
	signiture(argv[0]);

	gettimeofday(&t1, NULL);
	for (i = 0; i < NUM_PEOPLE; i += 1) {
		type = rand_type();
		start = rand_start_floor();
		dest = rand_dest_floor(start);
		fprintf(stdout, "Attempting to add a person of type %d, to floor %d, going to %d\n", type, start, dest);
		ret = issue_request(type, start, dest);
		if (ret == 0) {
			fprintf(stdout, "Successfully added\n");
		}
		else if (ret == 1) {
			fprintf(stderr, "Invalid request!\n");
			exit(1);
		}
		else {
			fprintf(stderr, "Module is not loaded!\n");
			exit(1);
		}
	}
	gettimeofday(&t2, NULL);

	fprintf(stdout, "Done adding people\n");

	time_diff(&t1, &t2, &diff);
	sleep_time = WAIT_TIME - diff.tv_sec;

	fprintf(stdout, "Waiting for the remaining %lld seconds\n", sleep_time);

	usleep(sleep_time * 1000000);

	fprintf(stdout, "Time is up!!!\n");
	
	return 0;
}
