#include <linux/init.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/kthread.h>
#include <linux/delay.h>
#include <asm-generic/uaccess.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Britton");
MODULE_DESCRIPTION("Simple module featuring proc read");

#define ENTRY_NAME "counter"
#define PERMS 0644
#define PARENT NULL
static struct file_operations fops;

static struct task_struct *kthread;
static int counter;
static char *message;
static int read_p;

int counter_run(void *data) {
	/* lock... */
	while (!kthread_should_stop()) {
		ssleep(1);
		counter += 1;

		/* unlock...*/
		/* schedule(); */
		/* lock... */
	}
	/* unlock... */
	printk("The counter thread has terminated\n");
	return counter;
	
}

int counter_proc_open(struct inode *sp_inode, struct file *sp_file) {
	printk("proc called open\n");
	
	read_p = 1;
	message = kmalloc(sizeof(char) * 20, __GFP_WAIT | __GFP_IO | __GFP_FS);
	if (message == NULL) {
		printk("ERROR, counter_proc_open");
		return -ENOMEM;
	}
	sprintf(message, "The counter is now at: %d\n", counter);
	return 0;
}

ssize_t counter_proc_read(struct file *sp_file, char __user *buf, size_t size, loff_t *offset) {
	int len = strlen(message);

	read_p = !read_p;
	if (read_p) {
		return 0;
	}
	
	printk("proc called read\n");
	copy_to_user(buf, message, len);
	return len;
}

int counter_proc_release(struct inode *sp_inode, struct file *sp_file) {
	printk("proc called release\n");
	kfree(message);
	return 0;
}



static int counter_init(void) {
	printk("/proc/%s create\n", ENTRY_NAME); 

	kthread = kthread_run(counter_run, NULL, "counter");
	if (IS_ERR(kthread)) {
		printk("ERROR! kthread_run\n");
		return PTR_ERR(kthread);
	}

	fops.open = counter_proc_open;
	fops.read = counter_proc_read;
	fops.release = counter_proc_release;
	
	if (!proc_create(ENTRY_NAME, PERMS, NULL, &fops)) {
		printk("ERROR! proc_create\n");
		remove_proc_entry(ENTRY_NAME, NULL);
		return -ENOMEM;
	}
	return 0;
}

static void counter_exit(void) {
	int ret = kthread_stop(kthread);
	if (ret != -EINTR)
		printk("Counter thread has stopped\n");
	remove_proc_entry(ENTRY_NAME, NULL);
	printk("Removing /proc/%s.\n", ENTRY_NAME);
}

module_init(counter_init);
module_exit(counter_exit);
